# Enise widgets for guizero
from enisegui.table         import Table
from enisegui.autotextboxkv import AutoTextBoxKV
from enisegui.datetextbox   import DateTextBox
from enisegui.combokv       import ComboKV
from enisegui.listboxkv     import ListBoxKV

# test code
def Demo():
    from guizero import App,Box,Text,ButtonGroup,PushButton
    import pkg_resources 
    
    curdir = pkg_resources.resource_filename('enisegui','.')

    sep_pad = 2
    app   = App(width=650, height=560) 
    mainbox = Box(app, width='fill')
    Box(mainbox, width='fill', height=sep_pad)
    
    # champ autocomplété   
    def choicelist(pattern):
        return [w for w in autolist if pattern in w]
        
    def choicelistKV(pattern):
        return [autolistKV[i] for i in range(len(autolistKV))
                        if pattern in autolistKV[i][0]]
    def auto_get(key):
        if key:
            autoval.value = key
        else:
            autoval.value = 'None'
        
    with open(curdir+'/autolist.txt') as file:
        autolist = file.read()
    autolist   = autolist.split('\n')
    autolistKV = [(autolist[i],'key-'+str(i)) for i in range(len(autolist))]

    autobox = Box(app, width='fill')
    Box(autobox, height='fill', width=4, align='left')
    autotext = Text(autobox, text='AutoTextBoxKV', width=18, align='left')
    autotext.tk.config(font='Helvetica 12 bold')
    autotext.tk.config(anchor='w')
    auto   = AutoTextBoxKV(autobox, width=20, max=6, text='Holmes Scott',
                            change=choicelistKV, command=auto_get, align='left')
    Box(autobox, width=4, height='fill', align='left')
    Text(autobox, text=' Key : ', align='left')
    autoval = Text(autobox, text='--', bg='green', color='white', align='left')
    auto.focus()
    
    # séparateur
    mainbox = Box(app, width='fill')
    Box(mainbox, width='fill', height=sep_pad)
    box = Box(mainbox, width='fill')
    Box(box, height='fill', width=4, align='left')
    sep = Box(box, width='fill', height=2, align='left')
    sep.set_border(1, 'grey50')
    Box(box, height=10, width=4, align='left')
    Box(mainbox, width='fill', height=sep_pad)
    
    # ComboKV
    def combo_get(key):
        combo_label.value  = key
        combo2.value = key
        
    def combo_set(value):
        combo.selectkey(value)
        
    box    = Box(app,width='fill')
    Box(box, height='fill', width=4, align='left')
    autotext = Text(box, text='ComboKV', width=18, align='left')
    autotext.tk.config(font='Helvetica 12 bold')
    autotext.tk.config(anchor='w')
    combo  = ComboKV(box, align='left', width=5,
                        options=[('Green', 12),('Yellow', 25),('Blue', 5)], 
                        command=combo_get)
    Text(box, align='left', text=' Key get ')
    combo_label  = Text(box, align='left', text='--', width=2, bg='green', color='white')
    Text(box, align='left', text=' Key set ')    
    combo2 = ComboKV(box, align='left', width=2, options=[12,25,5], command=combo_set)

    # séparateur
    mainbox = Box(app, width='fill')
    Box(mainbox, width='fill', height=sep_pad)
    box = Box(mainbox, width='fill')
    Box(box, height='fill', width=4, align='left')
    sep = Box(box, width='fill', height=2, align='left')
    sep.set_border(1, 'grey50')
    Box(box, height=10, width=4, align='left')
    Box(mainbox, width='fill', height=sep_pad)
    
    # ListBoxKV
    def listbox_get(key):
        listbox_label.value  = key
        listbox2.value = key
        
    def listbox_set(value):
        listbox.selectkey(value)
        
    box    = Box(app,width='fill')
    Box(box, height='fill', width=4, align='left')
    autotext = Text(box, text='ListBoxKV', width=18, align='left')
    autotext.tk.config(font='Helvetica 12 bold')
    autotext.tk.config(anchor='w')
    listbox  = ListBoxKV(box, align='left', width=100, height=60,
                        items=[('Green', 12),('Yellow', 25),('Blue', 5)], 
                        command=listbox_get)
    Text(box, align='left', text=' Key get ')
    listbox_label  = Text(box, align='left', text='--', width=2, bg='green', color='white')
    Text(box, align='left', text=' Key set ')    
    listbox2 = ListBoxKV(box, align='left', width=100, height=60,
                    items=[12, 25, 5], command=listbox_set)
    
    # séparateur
    mainbox = Box(app, width='fill')
    Box(mainbox, width='fill', height=sep_pad)
    box = Box(mainbox, width='fill')
    Box(box, height='fill', width=4, align='left')
    sep = Box(box, width='fill', height=2, align='left')
    sep.set_border(1, 'grey50')
    Box(box, height=10, width=4, align='left')
    Box(mainbox, width='fill', height=sep_pad)
    
    # date picker
    datebox = Box(app, width='fill')
    Box(datebox, height='fill', width=4, align='left')
    datetext = Text(datebox, text='DateTextBox', width=12, align='left')
    datetext2 = Text(datebox, text='(unfilled)', width=10, align='left')
    datetext.tk.config(font='Helvetica 12 bold')
    datetext.tk.config(anchor='w')
    datetext2.tk.config(anchor='w')
    date1 = DateTextBox(datebox, width=20, align='left')   # empty one    
    def date1_set():
        date1.value=d1.value
    PushButton(datebox, align='left', text='Set', command=date1_set)
    d1 = Text(datebox, text='1998-01-05', bg='blue', color='white', align='left')
    
    datebox = Box(app, width='fill')
    Box(datebox, height='fill', width=4, align='left')
    datetext = Text(datebox, text='DateTextBox', width=12, align='left')
    datetext2 = Text(datebox, text='(prefilled)', width=10, align='left')
    datetext.tk.config(font='Helvetica 12 bold')
    datetext.tk.config(anchor='w')
    datetext2.tk.config(anchor='w')
    date2 = DateTextBox(datebox, width=20, 
                        text='2012-05-08',    # pre-filled one
                        align='left')
    def date2_get():
        d2.value=date2.value
    PushButton(datebox, align='left', text='Get', command=date2_get)
    d2 = Text(datebox, text='--', bg='green', color='white', align='left')
    
    # séparateur
    mainbox = Box(app, width='fill')
    Box(mainbox, width='fill', height=sep_pad)
    box = Box(mainbox, width='fill')
    Box(box, height='fill', width=4, align='left')
    sep = Box(box, width='fill', height=2, align='left')
    sep.set_border(1, 'grey50')
    Box(box, height=10, width=4, align='left')
    Box(mainbox, width='fill', height=sep_pad)
    
    # tables
    data1 = [                               # liste de tuples
        ('id','name','price'),
        (251,'Orange',12.5),
        (6852,'Potatoe',1220),
        (31,'Fruit',858.5)
    ]
    
    data2 = [                 # liste de dictionnaires sans rowid
        {'id':75,'name':'Kiwi','price':3.5,'family':'fruit'},
        {'id':542,'name':'Tomatoe','price':7.50,'family':'vegetable'},
        {'id':12,'name':'Fuji','price':4.0,'family':'apple'}
    ]        
    
    data3 = [                 # liste de dictionnaires avec rowid
        {'rowid':75,'name':'Kiwi','price':3.5,'family':'fruit'},
        {'rowid':542,'name':'Tomatoe','price':7.50,'family':'vegetable'},
        {'rowid':12,'name':'Fuji','price':4.0,'family':'apple'}
    ]      
    
    def to_rowid(list):
        for i in range(len(list)):
            dict = list[i]
            if 'id' in dict:
                rowid = dict['id']
                del dict['id']
                dict['rowid'] = rowid
        return list
            
    def to_id(list):
        for i in range(len(list)):
            dict = list[i]
            if 'rowid' in dict:
                id = dict['rowid']
                del dict['rowid']
                dict['id'] = id
        return list
    
    # callbacks
    def table_action(num):
        label.value = str(num)
        
    def table_fill():
        command = {"1":None,  "2":table_action}
        rowid   = {"1":False, "2":True}
        if choose_data.value=="1":
            data = data1
        elif (choose_data.value, choose_rowid.value)==("2","1"):
            data = to_id(data2)
        elif (choose_data.value, choose_rowid.value)==("2","2"):
            data = to_rowid(data2)
        table.rowid = rowid[choose_rowid.value]
        table.data  = data
        table.update_command(command[choose_command.value])
        label.value   = "--"
    # panneau de contrôle
    panneau        = Box(app, width='fill')
    topbox         = Box(panneau, width='fill')
    Box(topbox, height='fill', width=4, align='left')
    tabletext = Text(topbox, text='Table', align='left')
    tabletext.tk.config(font='Helvetica 12 bold')
    mainbox        = Box(panneau, width='fill')
    choose_data    = ButtonGroup(mainbox, 
                            options=[["tuples", "1"],["dicts", "2"]],
                            selected="1", 
                            command=table_fill, align='left')
    choose_rowid   = ButtonGroup(mainbox, 
                            options=[["no rowid", "1"],["rowid", "2"]],
                            selected="2", 
                            command=table_fill, align='left')
    choose_command = ButtonGroup(mainbox, 
                            options=[["no command", "1"],["command", "2"]],
                            selected="1", 
                            command=table_fill, align='left')
    Text(mainbox, text='Line # : ', align='left')
    label  = Text(mainbox, text='--', bg='green', color='white', align='left')
    
    # zone d'affichage pour la table
    affichage   = Box(app, width='fill')
    topbox      = Box(affichage, width='fill', height=10)
    mainbox     = Box(affichage, width='fill')
    Box(mainbox, align='left', height='fill', width=10)
    table       = Table(mainbox, data=data1, align='left', rowid=True)
    
    
    app.display() 




from guizero   import Box
from tkinter   import StringVar
from .autoentry import AutoEntry
        
class AutoTextBoxKV(Box):
    def __init__(self, parent=None, autolist=[], **kwargs):
        # starting parameters
        text    = kwargs.pop('text', '')
        change  = kwargs.pop('change', None)
        command = kwargs.pop('command', None)
        max     = kwargs.pop('max', 8)
        width   = kwargs.pop('width', 10)
        # initialize guizero box
        super(AutoTextBoxKV,self).__init__(parent, **kwargs)
        # attach tkinter widget inside self.tk Frame
        textvariable = StringVar()
        textvariable.set(text)
        self._entry = AutoEntry(self.tk, autolist, textvariable=textvariable,
                            change=change, command=command, max=max, width=width)
        self._entry.pack()
        
    def focus(self):
        # set focus to entry widget
        self._entry.focus()    
        
    def update_command(self, command):
        # update command callback
        self._entry.update_command(command)
        
    def update_change(self, change):
        # update change callback
        self._entry.update_change(change)
        
    @property
    def key(self):
        # get key
        return self._entry.key
        
    @property
    def value(self):
        # get value
        return self._entry.value

    @value.setter
    def value(self, value):
        # set value
        self._entry.value = value
        
       
# test code
if __name__ == '__main__':
    from guizero import App,Box,Text
            
    # autocomplete callbacks   
    def choicelist(pattern):
        return [w for w in autolist if pattern in w]
        
    def choicelistKV(pattern):
        return [autolistKV[i] for i in range(len(autolistKV))
                        if pattern in autolistKV[i][0]]
    def auto_get(key):
        if key:
            autoval.value = key
        else:
            autoval.value = 'None'
 
    with open('autolist.txt') as file:
        autolist = file.read()
    autolist   = autolist.split('\n')
    autolistKV = [(autolist[i],'key-'+str(i)) for i in range(len(autolist))]

    app    = App(height=100) 
    autobox = Box(app, width='fill')
    Box(autobox, height='fill', width=4, align='left')
    autotext = Text(autobox, text='AutoTextBoxKV', width=18, align='left')
    autotext.tk.config(font='Helvetica 12 bold')
    autotext.tk.config(anchor='w')
    auto   = AutoTextBoxKV(autobox, width=20, max=6, text='Holmes Scott',
                            change=choicelistKV, command=auto_get, align='left')
    Box(autobox, width=4, height='fill', align='left')
    Text(autobox, text=' Key : ', align='left')
    autoval = Text(autobox, text='--', bg='green', color='white', align='left')
    auto.focus()
    
    app.display()

  
        


# modded and debugged version from source
# http://code.activestate.com/recipes/580725-tkinter-datepicker-like-the-jquery-ui-datepicker/
# Original author: Miguel Martinez Lopez

"""
These are the default bindings:
    Get focus : Show calendar
    Click button 1 on entry: Show calendar
    Click button 1 outsite calendar and entry: Hide calendar
    Escape: Hide calendar
    CTRL + PAGE UP: Move to the previous month.
    CTRL + PAGE DOWN: Move to the next month.
    CTRL + SHIFT + PAGE UP: Move to the previous year.
    CTRL + SHIFT + PAGE DOWN: Move to the next year.
    CTRL + LEFT: Move to the previous day.
    CTRL + RIGHT: Move to the next day.
    CTRL + UP: Move to the previous week.
    CTRL + DOWN: Move to the next week.
    CTRL + END: Close the datepicker and erase the date.
    CTRL + HOME: Move to the current month.
    CTRL + SPACE: Show date on calendar
    CTRL + Return: Set current selection to entry
"""

import calendar
import datetime

try:
    import Tkinter
    import tkFont
    import ttk

    from Tkconstants import CENTER, LEFT, RIGHT, N, E, W, S
    from Tkinter import StringVar
except ImportError: # py3k
    import tkinter as Tkinter
    import tkinter.font as tkFont
    import tkinter.ttk as ttk

    from tkinter.constants import CENTER, LEFT, RIGHT, N, E, W, S
    from tkinter import StringVar

def get_calendar(locale, fwday):
    # instantiate proper calendar class
    if locale is None:
        return calendar.TextCalendar(fwday)
    else:
        return calendar.LocaleTextCalendar(fwday, locale)


class Calendar(ttk.Frame):
    datetime = calendar.datetime.datetime
    timedelta = calendar.datetime.timedelta

    def __init__(self, master=None, year=None, month=None, firstweekday=calendar.MONDAY, locale=None, activebackground='#b1dcfb', activeforeground='black', selectbackground='#003eff', selectforeground='white', command=None, select=None, borderwidth=1, relief="solid", on_click_month_button=None):
        """
        WIDGET OPTIONS

            locale, firstweekday, year, month, selectbackground,
            selectforeground, activebackground, activeforeground, 
            command, borderwidth, relief, on_click_month_button
        """

        if year is None:
            year = self.datetime.now().year
        
        if month is None:
            month = self.datetime.now().month

        self._selected_date = None

        self._sel_bg = selectbackground 
        self._sel_fg = selectforeground

        self._act_bg = activebackground 
        self._act_fg = activeforeground
        
        self._norm_bg = "white"
        self._norm_fg = "grey30"
        
        self.on_click_month_button = on_click_month_button
        
        self._selection_is_visible = False
        self._command = command
        self._select  = select

        ttk.Frame.__init__(self, master, borderwidth=borderwidth, relief=relief)
        
        self.bind("<FocusIn>", lambda event:self.event_generate('<<DatePickerFocusIn>>'))
        self.bind("<FocusOut>", lambda event:self.event_generate('<<DatePickerFocusOut>>'))
    
        self._cal = get_calendar(locale, firstweekday)

        # custom ttk styles
        style = ttk.Style()
        style.layout('L.TButton', (
            [('Button.focus', {'children': [('Button.leftarrow', None)]})]
        ))
        style.layout('R.TButton', (
            [('Button.focus', {'children': [('Button.rightarrow', None)]})]
        ))

        self._font = tkFont.Font()
        
        self._header_var = StringVar()
        self._year_var   = StringVar()

        # header frame and its widgets
        hframe = ttk.Frame(self)
        lbtn = ttk.Button(hframe, style='L.TButton', command=self._on_press_year_left_button)
        lbtn.grid(row=0,column=0,sticky=E)
        
        datefont = tkFont.Font(weight='bold')
        self._header = ttk.Label(hframe, font=datefont,  anchor=CENTER,
         textvariable=self._year_var)
        self._header.grid(row=0,column=1)
        rbtn = ttk.Button(hframe, style='R.TButton', command=self._on_press_year_right_button)
        rbtn.grid(row=0,column=2,sticky=W)
        
        hframe2 = ttk.Frame(self)
        lbtn2 = ttk.Button(hframe, style='L.TButton',
                        command=self._on_press_left_button)
        lbtn2.grid(row=1,column=0)
        
        self._header2 = ttk.Label(hframe, width=12, anchor=CENTER,
                            textvariable=self._header_var)
        self._header2.grid(row=1,column=1)
        
        rbtn2 = ttk.Button(hframe, style='R.TButton',
                        command=self._on_press_right_button)
        rbtn2.grid(row=1,column=2)

        hframe.grid(row=0,column=0,columnspan=7,pady=2)
        
        self._day_labels = {}

        days_of_the_week = self._cal.formatweekheader(3).split()
 
        for i, day_of_the_week in enumerate(days_of_the_week):
            Tkinter.Label(self, 
                    text=day_of_the_week, 
                    background='grey90').grid(row=2, column=i, sticky=N+E+W+S)

        for i in range(6):
            for j in range(7):
                self._day_labels[i,j] = label = Tkinter.Label(self, 
                            background = self._norm_bg,
                            foreground = self._norm_fg)
                
                label.grid(row=i+3, column=j, sticky=N+E+W+S)
                label.bind("<Enter>", 
                    lambda event: event.widget.configure(background=self._act_bg, 
                        foreground=self._act_fg))
                label.bind("<Leave>", 
                    lambda event: event.widget.configure(background=self._norm_bg,
                        foreground=self._norm_fg))

                label.bind("<1>", self._pressed)
        
        # adjust its columns width
        font = tkFont.Font()
        maxwidth = max(font.measure(text) for text in days_of_the_week)
        for i in range(7):
            self.grid_columnconfigure(i, minsize=maxwidth, weight=1)

        self._year = None
        self._month = None

        # insert dates in the currently empty calendar
        self._build_calendar(year, month)

    def _build_calendar(self, year, month):
        if not( self._year == year and self._month == month):
            self._year = year
            self._month = month

            # update header text (Month, YEAR)
            header = self._cal.formatmonthname(year, month, 1, False)
            theyear   = self._cal.formatyear(year, 0)
            self._header_var.set(header.title())
            self._year_var.set(year)

            # update calendar shown dates
            cal = self._cal.monthdayscalendar(year, month)

            for i in range(len(cal)):
                
                week = cal[i] 
                fmt_week = [('%02d' % day) if day else '' for day in week]
                
                for j, day_number in enumerate(fmt_week):
                    self._day_labels[i,j]["text"] = day_number

            if len(cal) < 6:
                for j in range(7):
                    self._day_labels[5,j]["text"] = ""

        if self._selected_date is not None and self._selected_date.year == self._year and self._selected_date.month == self._month:
            self._show_selection()

    def _find_label_coordinates(self, date):
        # original code was quite buggy, for exemple for month april of 2019
        # first_weekday_of_the_month = (date.weekday() - date.day) % 7
        # return divmod((first_weekday_of_the_month-1 - self._cal.firstweekday)%7 + date.day, 7)
        
        # new code seems to work much better, though I don't know exactly why ;-)
        first_weekday_of_the_month = datetime.date(date.year,date.month,1).weekday()
        if date.weekday()!=6:
            a=divmod((first_weekday_of_the_month - self._cal.firstweekday)%7 + date.day, 7)
            b=divmod((first_weekday_of_the_month -1 - self._cal.firstweekday)%7 + date.day, 7)
            return (min(a[0],b[0]),b[1])
        else:
            a=divmod((first_weekday_of_the_month - self._cal.firstweekday)%7 + date.day-1, 7)
            b=divmod((first_weekday_of_the_month -1 - self._cal.firstweekday)%7 + date.day-1, 7)
            return (min(a[0],b[0]),b[1]+1)
        
    def _show_selection(self):
        """Show a new selection."""

        i,j = self._find_label_coordinates(self._selected_date)

        label = self._day_labels[i,j]

        label.configure(background=self._sel_bg, foreground=self._sel_fg)

        label.unbind("<Enter>")
        label.unbind("<Leave>")
        
        self._selection_is_visible = True
        
    def _clear_selection(self):
        """Show a new selection."""
        i,j = self._find_label_coordinates(self._selected_date)

        label = self._day_labels[i,j]
        label.configure(background=self._norm_bg, foreground=self._norm_fg)

        label.bind("<Enter>", 
            lambda event: event.widget.configure(background=self._act_bg, 
                foreground=self._act_fg))
        label.bind("<Leave>", 
            lambda event: event.widget.configure(background=self._norm_bg, 
                foreground=self._norm_fg))

        self._selection_is_visible = False

    # Callback

    def _pressed(self, evt):
        """Clicked somewhere in the calendar."""
        
        text = evt.widget["text"]
        
        if text == "":
            return

        day_number = int(text)

        new_selected_date = datetime.datetime(self._year, self._month, day_number)
        if self._selected_date != new_selected_date:
            if self._selected_date is not None:
                self._clear_selection()
            
            self._selected_date = new_selected_date
            if self._select:
                self._select(self._selected_date)
            
            self._show_selection()
        
        if self._command:
            self._command(self._selected_date)

    def _on_press_year_left_button(self):
        self.prev_year()
        
        if self.on_click_month_button is not None:
            self.on_click_month_button()

    def _on_press_left_button(self):
        self.prev_month()
        
        if self.on_click_month_button is not None:
            self.on_click_month_button()
    
    def _on_press_year_right_button(self):
        self.next_year()

        if self.on_click_month_button is not None:
            self.on_click_month_button()
    
    def _on_press_right_button(self):
        self.next_month()

        if self.on_click_month_button is not None:
            self.on_click_month_button()
        
    def select_prev_day(self):
        """Updated calendar to show the previous day."""
        if self._selected_date is None:
            self._selected_date = datetime.datetime(self._year, self._month, 1)
        else:
            self._clear_selection()
            self._selected_date = self._selected_date - self.timedelta(days=1)
            
        if self._select:
            self._select(self._selected_date)

        self._build_calendar(self._selected_date.year, self._selected_date.month) # reconstruct calendar

    def select_next_day(self):
        """Update calendar to show the next day."""

        if self._selected_date is None:
            self._selected_date = datetime.datetime(self._year, self._month, 1)
        else:
            self._clear_selection()
            self._selected_date = self._selected_date + self.timedelta(days=1)
            
        if self._select:
            self._select(self._selected_date)

        self._build_calendar(self._selected_date.year, self._selected_date.month) # reconstruct calendar


    def select_prev_week_day(self):
        """Updated calendar to show the previous week."""
        if self._selected_date is None:
            self._selected_date = datetime.datetime(self._year, self._month, 1)
        else:
            self._clear_selection()
            self._selected_date = self._selected_date - self.timedelta(days=7)
            
        if self._select:
            self._select(self._selected_date)

        self._build_calendar(self._selected_date.year, self._selected_date.month) # reconstruct calendar

    def select_next_week_day(self):
        """Update calendar to show the next week."""
        if self._selected_date is None:
            self._selected_date = datetime.datetime(self._year, self._month, 1)
        else:
            self._clear_selection()
            self._selected_date = self._selected_date + self.timedelta(days=7)
            
        if self._select:
            self._select(self._selected_date)

        self._build_calendar(self._selected_date.year, self._selected_date.month) # reconstruct calendar

    def select_current_date(self):
        """Update calendar to current date."""
        if self._selection_is_visible: self._clear_selection()

        self._selected_date = datetime.datetime.now()
            
        if self._select:
            self._select(self._selected_date)
            
        self._build_calendar(self._selected_date.year, self._selected_date.month)

    def prev_month(self):
        """Updated calendar to show the previous week."""
        if self._selection_is_visible: self._clear_selection()
        
        date = self.datetime(self._year, self._month, 1) - self.timedelta(days=1)
        self._selected_date = date
            
        if self._select:
            self._select(self._selected_date)
            
        self._build_calendar(date.year, date.month) # reconstuct calendar

    def next_month(self):
        """Update calendar to show the next month."""
        if self._selection_is_visible: self._clear_selection()

        date = self.datetime(self._year, self._month, 1) + \
            self.timedelta(days=calendar.monthrange(self._year, self._month)[1] + 1)
        self._selected_date = date
            
        if self._select:
            self._select(self._selected_date)

        self._build_calendar(date.year, date.month) # reconstuct calendar

    def prev_year(self):
        """Updated calendar to show the previous year."""
        
        if self._selection_is_visible: self._clear_selection()

        date = self.datetime(self._year-1, self._month,1)
        self._selected_date = date
            
        if self._select:
            self._select(self._selected_date)

        self._build_calendar(date.year, date.month) # reconstruct calendar

    def next_year(self):
        """Update calendar to show the next year."""
        
        if self._selection_is_visible: self._clear_selection()

        date = self.datetime(self._year+1, self._month,1)
        self._selected_date = date
            
        if self._select:
            self._select(self._selected_date)

        self._build_calendar(date.year, date.month) # reconstruct calendar

    def get_selection(self):
        """Return a datetime representing the current selected date."""
        return self._selected_date
        
    selection = get_selection

    def set_selection(self, date):
        """Set the selected date."""
        if self._selected_date is not None and self._selected_date != date:
            self._clear_selection()

        self._selected_date = date
            
        if self._select:
            self._select(self._selected_date)

        self._build_calendar(date.year, date.month) # reconstruct calendar

# see this URL for date format information:
#     https://docs.python.org/3/library/datetime.html#strftime-strptime-behavior

class Datepicker(ttk.Entry):
    def __init__(self, master, entrywidth=None, entrystyle=None, datevar=None, dateformat="%Y-%m-%d", onselect=None, firstweekday=calendar.MONDAY, locale=None, activebackground='#b1dcfb', activeforeground='black', selectbackground='#003eff', selectforeground='white', borderwidth=1, relief="solid"):
        
        if datevar is not None:
            self.date_var = datevar
        else:
            self.date_var = Tkinter.StringVar()
        
        entry_config = {}
        if entrywidth is not None:
            entry_config["width"] = entrywidth
            
        if entrystyle is not None:
            entry_config["style"] = entrystyle
    
        ttk.Entry.__init__(self, master, textvariable=self.date_var, **entry_config)

        self.date_format = dateformat
        
        self._is_calendar_visible = False
        self._on_select_date_command = onselect

        self.calendar_frame = Calendar(self.winfo_toplevel(), firstweekday=firstweekday, locale=locale, activebackground=activebackground, activeforeground=activeforeground, selectbackground=selectbackground, selectforeground=selectforeground, command=self._on_selected_date, select=self._on_select_date, on_click_month_button=lambda: self.focus())

        self.bind_all("<1>", self._on_click, "+")

        self.bind("<FocusOut>", lambda event: self._on_entry_focus_out())
        self.bind("<Escape>", lambda event: self.esc_hide_calendar())
        self.calendar_frame.bind("<<DatePickerFocusOut>>", lambda event: self._on_calendar_focus_out())


        # CTRL + PAGE UP: Move to the previous month.
        self.bind("<Control-Prior>", lambda event: self.calendar_frame.prev_month())
        
        # CTRL + PAGE DOWN: Move to the next month.
        self.bind("<Control-Next>", lambda event: self.calendar_frame.next_month())

        # CTRL + SHIFT + PAGE UP: Move to the previous year.
        self.bind("<Control-Shift-Prior>", lambda event: self.calendar_frame.prev_year())

        # CTRL + SHIFT + PAGE DOWN: Move to the next year.
        self.bind("<Control-Shift-Next>", lambda event: self.calendar_frame.next_year())
        

        # CTRL + LEFT: Move to the previous day.
        # self.bind("<Control-Left>", lambda event: self.calendar_frame.select_prev_day())
        def ctrl_left(event):
            self.calendar_frame.select_prev_day()
            return 'break'
        self.bind("<Control-Left>", ctrl_left)
        
        # CTRL + RIGHT: Move to the next day.
        # self.bind("<Control-Right>", lambda event: self.calendar_frame.select_next_day())
        def ctrl_right(event):
            self.calendar_frame.select_next_day()
            return 'break'
        self.bind("<Control-Right>", ctrl_right)
        
        # CTRL + UP: Move to the previous week.
        self.bind("<Control-Up>", lambda event: self.calendar_frame.select_prev_week_day())
        
        # CTRL + DOWN: Move to the next week.
        self.bind("<Control-Down>", lambda event: self.calendar_frame.select_next_week_day())

        # CTRL + END: Close the datepicker and erase the date.
        self.bind("<Control-End>", lambda event: self.erase())

        # CTRL + HOME: Move to the current month.
        self.bind("<Control-Home>", lambda event: self.calendar_frame.select_current_date())
        
        # CTRL + SPACE: Show date on calendar
        self.bind("<Control-space>", lambda event: self.show_date_on_calendar())
        
        # CTRL + Return: Set to entry current selection
        self.bind("<Control-Return>", lambda event: self.set_date_from_calendar())
        
        # MOD on focus
        self.bind("<FocusIn>", self._on_click, "+")
        
        if self.current_text=='':
            self.calendar_frame.select_current_date()

    def set_date_from_calendar(self):
        if self.is_calendar_visible:
            selected_date = self.calendar_frame.selection()

            if selected_date is not None:
                self.date_var.set(selected_date.strftime(self.date_format))
                
                if self._on_select_date_command is not None:
                    self._on_select_date_command(selected_date)

            self.hide_calendar()
      
    @property
    def current_text(self):
        return self.date_var.get()
        
    @current_text.setter
    def current_text(self, text):
        return self.date_var.set(text)
        
    @property
    def current_date(self):
        try:
            date = datetime.datetime.strptime(self.date_var.get(), self.date_format)
            return date
        except ValueError:
            return None
    
    @current_date.setter
    def current_date(self, date):
        self.date_var.set(date.strftime(self.date_format))
        
    @property
    def is_valid_date(self):
        if self.current_date is None:
            return False
        else:
            return True

    def show_date_on_calendar(self):
        date = self.current_date
        if date is not None:
            self.calendar_frame.set_selection(date)

        self.show_calendar()

    def show_calendar(self):
        self._old_date = self.current_text
        if not self._is_calendar_visible:
            self.calendar_frame.place(in_=self, relx=0, rely=1)
            self.calendar_frame.lift()

        self._is_calendar_visible = True

    def hide_calendar(self):
        if self._is_calendar_visible:
            self.calendar_frame.place_forget()
        
        self._is_calendar_visible = False

    def esc_hide_calendar(self):
        self.date_var.set(self._old_date)
        if self._is_calendar_visible:
            self.calendar_frame.place_forget()
        
        self._is_calendar_visible = False

    def erase(self):
        self.hide_calendar()
        self.date_var.set("")
    
    @property
    def is_calendar_visible(self):
        return self._is_calendar_visible

    def _on_entry_focus_out(self):
        if not str(self.focus_get()).startswith(str(self.calendar_frame)):
            self.hide_calendar()
        
    def _on_calendar_focus_out(self):
        if self.focus_get() != self:
            self.hide_calendar()

    def _on_selected_date(self, date):
        self.date_var.set(date.strftime(self.date_format))
        self.hide_calendar()
        
        if self._on_select_date_command is not None:
            self._on_select_date_command(date)

    def _on_select_date(self, date):
        self.date_var.set(date.strftime(self.date_format))
        
        if self._on_select_date_command is not None:
            self._on_select_date_command(date)

    def _on_click(self, event):
        str_widget = str(event.widget)

        if str_widget == str(self):
            if not self._is_calendar_visible:
                self.show_date_on_calendar()
        else:
            if not str_widget.startswith(str(self.calendar_frame)) and self._is_calendar_visible:
                self.hide_calendar()

# test code
if __name__ == "__main__":
    import sys
    if sys.platform == 'linux':
        locale = 'fr_FR.UTF-8'
    else:
        locale = 'French_France.1252'

    try:
        from Tkinter import Tk, Frame, Label
    except ImportError:
        from tkinter import Tk, Frame, Label
    
    root = Tk()
    root.geometry("500x600")
    
    main =Frame(root, pady =15, padx=15)
    main.pack(expand=True, fill="both")
    
    Label(main, justify="left", text=__doc__).pack(anchor="w", pady=(0,15))
    e = Datepicker(main,locale=locale)
    e.pack(anchor="w")
    e.focus()
    
    var = StringVar()
    var.set('2018-01-21')
    e = Datepicker(main,locale=locale, datevar=var)
    e.pack(anchor="w")
    
    if 'win' not in sys.platform:
        style = ttk.Style()
        style.theme_use('clam')

    root.mainloop()



"""
Inspired by https://gist.github.com/uroshekic/11078820
who was himself inspired by 
http://code.activestate.com/recipes/578253-an-entry-with-autocompletion-for-the-tkinter-gui/
"""

from tkinter import Tk,Entry,StringVar,Listbox, Frame
from tkinter.constants import END, INSERT, ACTIVE
from tkinter import sys

class AutoEntry(Entry):
    def __init__(self, master, *args, **kwargs):
        # paramètres
        self._max       = kwargs.pop('max', 6)
        self._change    = kwargs.pop('change', None)
        self._command   = kwargs.pop('command', None)
        self._text      = kwargs.pop('textvariable')
        
        # réglage des couleurs de sélection pour la listbox
        if sys.platform == 'linux':
            self._highlight = { 'selectbackground':'medium blue',
                                'selectforeground':'white'}
        else:
            self._highlight = {}
        kwargs.update(self._highlight)
        
        # initialisation du champs de saisie
        super(AutoEntry,self).__init__(master,*args,
                                        textvariable=self._text,**kwargs)
        self.icursor(END)

        # la listbox de choix n'est pas visible
        self._listboxUp = False          
        
        # type de liste reçue : liste de tuples ou liste de valeurs
        if self._change and self._text.get():
            choices = self._change(self._text.get())
            if choices:
                if type(choices[0])!=tuple:
                    self._KV     = False
                    self._values = choices
                else:
                    self._KV = True
                    self._keys   = [str(t[1]) for t in choices]
                    self._values = [str(t[0]) for t in choices]
        else:
            choices = []
            self._KV = False
            
        # définition des callbacks
        # self._text.trace('w', self._on_change) 
        self.bind("<KeyRelease>", self._on_keyrelease)
        self.bind("<Right>", self.select)
        self.bind("<Up>", self.moveUp)
        self.bind("<Down>", self.moveDown)
        self.bind("<Escape>", self.hide)
        self.bind("<FocusOut>", self.focusout)
        self.bind("<Control-Right>", self._show)
        self.bind_all("<Button-1>", self.click)
            
    def focusout(self, event):
        ''' disparition de la liste si le focus est perdu '''
        if self._listboxUp:
            self.hide(event)
        
    def click(self, event):
        ''' disparition de la liste si on clique à côté '''
        if self._listboxUp and event.widget!=self and event.widget!=self.listbox:
            self.hide(event)

    def mouse(self, event):
        ''' choix d'un item si on clique dans la liste '''
        if self.listbox.curselection():
            self.value = self.listbox.get(self.listbox.curselection()[0])
            self.icursor(END)
            self.hide(event)
        
    def select(self, event):
        ''' choix au clavier d'un item dans la liste '''
        if self._listboxUp:
            self.value = self.listbox.get(ACTIVE)
            self.icursor(END)
            self.hide(event)
        
    def hide(self, event):
        ''' disparition de la liste '''
        if self._listboxUp:
            self._listboxframe.destroy()
            self._listboxUp = False
                
    def _on_value(self):
        if self._command:
            if self._KV:
                self._command(self.key)
            else:
                self._command(self.value)
    
    def _show(self, event):
        self._on_keyrelease(event)
               
    def _on_keyrelease(self, event):
        if (len(event.keysym)==1 
            or event.keysym=="Delete" or event.keysym=="BackSpace"):
            if self.value == '':
                self.hide(event)
            elif self._change:
                choices = self._change(self._text.get())
                if choices:
                    if type(choices[0])!=tuple:
                        self._KV     = False
                        self._values = choices
                    else:
                        self._KV = True
                        self._keys   = [str(t[1]) for t in choices]
                        self._values = [str(t[0]) for t in choices]
    
                    if not self._listboxUp:        
                        self._listboxframe = Frame(self.winfo_toplevel())                   
                        self._listboxframe.place(in_=self, relx=0, rely=1)
                        self._listboxframe.lift()
                        self.listbox = Listbox(self._listboxframe, width=self["width"],
                                                height=self._max,
                                                **self._highlight)
                        self.listbox.pack()     
                        self.listbox.bind("<<ListboxSelect>>", self.mouse)
                        self._listboxUp = True
                    
                    self.listbox.delete(0, END)
                    for choice in self._values:
                        self.listbox.insert(END, choice)
                    self.listbox["height"] = min(len(choices), self._max)
                    self.goto(0)
                else:
                    self.hide(event)
                        
    def moveUp(self, event):
        if self._listboxUp:
            if self.index:                
                self.listbox.selection_clear(self.index)
                self.goto(self.index-1)

    def moveDown(self, event):
        if self._listboxUp:
            if self.index != self.listbox.size()-1:                        
                self.listbox.selection_clear(self.index)
                self.goto(self.index+1)
                
    def goto(self, index):
        self.index = index
        self.listbox.see(index)
        self.listbox.selection_set(index)
        self.listbox.activate(index)    
        
    @property
    def key(self):
        try:
            return self._keys[self._values.index(self.value)]
        except:
            return None
        
    @property
    def value(self):
        return self._text.get()
        
    @value.setter
    def value(self, value):
        self._text.set( str(value) )
        self._on_value()
                          
    def update_command(self, command):
        self._command = command    
                              
    def update_change(self, change):
        self._change = change

# test code
if __name__ == '__main__':
    from tkinter import Frame
    
    def choicelist(pattern):
        return [w for w in autolist if pattern in w]
        
    def choicelistKV(pattern):
        return [autolistKV[i] for i in range(len(autolistKV))
                        if pattern in autolistKV[i][1]]
        
    with open('autolist.txt') as file:
        autolist = file.read()
    autolist   = autolist.split('\n')
    autolistKV = [(autolist[i],'key-'+str(i)) for i in range(len(autolist))]
    
    root = Tk()
    root.title = "AutoEntry"
    root.geometry("300x200")
    text = StringVar()
    textKV = StringVar()
    frame = Frame(root)
    entry = AutoEntry(frame, textvariable=text, 
                    change=choicelist, max=8, width=20)
    frame2 = Frame(root)
    entryKV = AutoEntry(frame2, textvariable=textKV, 
                    change=choicelistKV, max=8, width=20)
    entry.focus()
    entry.pack()
    entryKV.pack()
    frame.pack()    
    frame2.pack()
    root.mainloop()



#import setuptools
from distutils.core import setup
setup(
      name = 'enisegui',
      packages = ['enisegui'],
      package_data = {'enisegui':['*.txt']},                    
      version = '0.0.2',
      description = 'Widgets for guizero',
      author = 'BagEddy42',
      author_email = 'edouard.vidal@enise.fr',
      keywords = ['gui', 'guizero'], # arbitrary keywords
      install_requires=[
          'guizero',
      ],
      classifiers = [],
)
